#define _SUPPRESS_PLIB_WARNING 1
#include "plib.h"
#include "pressure.h"
#include <math.h>
#include <GenericTypeDefs.h>


//WILL USE SPI CHANNEL 2 TO COMMUNICATE WITH PRESSURE

inline void Mode16f(void){  // configure SPI1 for 16-bit mode
    SPI2CONSET = 0x400;
    //SPI_MODE16_ON;
}

inline void Mode8f(void){  // configure SPI1 for 8-bit mode
    SPI2CONCLR = 0x400;
    //SPI_MODE16_OFF;
}

void pressure_begin(unsigned char mode) {
    
    /* Mode boundary check */
    if ((mode > BMP183_MODE_ULTRAHIGHRES) || (mode < 0))
    {
        mode = BMP183_MODE_ULTRAHIGHRES;
    }
    /* Set the mode indicator */
    oversampling = mode;
    
    //read out the calibration data
    delayMs(15);
    read_calibration_data();
    
}

void spiwrite(unsigned char addr, unsigned char data) {
    UINT16 msg = 0;
    msg = addr & 0x7F;
    msg = msg << 8;
    msg = msg | data;
    PR_CS = 0;
    Mode16f();
    while (TxBufFullSPI2());
    //or just combine this into two bytes - unsigned short
    WriteSPI2(msg); //binary AND with 01111111 (control byte) - 0 for write
    while (SPI2STATbits.SPIBUSY); // wait for it to end of transaction
    ReadSPI2();
    PR_CS = 1;
    Mode8f();
}

unsigned char spiread8(unsigned char addr) {
    unsigned char r;
    PR_CS = 0;
    while(TxBufFullSPI2());
    WriteSPI2(0x80 | addr); //binary OR with 10000000 (control byte) - 1 for read
    while (SPI2STATbits.SPIBUSY); // wait for it to end of transaction
    ReadSPI2();
    WriteSPI2(0);
    while (SPI2STATbits.SPIBUSY);
    r = ReadSPI2();
    PR_CS = 1;
    return r;
}

unsigned int spiread16(unsigned char addr) {
    unsigned int r;
    PR_CS = 0;
    while(TxBufFullSPI2());
    WriteSPI2(0x80 | addr);
    while (SPI2STATbits.SPIBUSY); // wait for it to end of transaction
    ReadSPI2();
    Mode16f(); // switch to 16-bit mode
    WriteSPI2(0);
    while (SPI2STATbits.SPIBUSY);
    r = ReadSPI2();
    PR_CS = 1;
    Mode8f(); // switch back to 8-bit mode
    return r;
}

/*
 Returns the temperature in degrees Celsius
 */
float get_temperature(void) {
    INT32 UT, X1, X2, B5;
    float temp;
    
    UT = read_raw_temperature();
    //do calculations on UT with the calibration data to get in deg C
    X1 = (UT - (INT32)AC6) * (INT32)AC5 >> 15;
    X2 = ((INT32)MC << 11) / (X1 + (INT32)MD);
    B5 = X1 + X2;
    temp = (B5 + 8) >> 4;
    temp /= 10;
    return temp;
}

/*
 Returns the pressure in hepto-Pascals (hPa)
 */
INT32 get_pressure(void) {
    INT32 UT, UP, B3, B5, B6, X1, X2, X3, p;
    UINT32 B4, B7;
    
    UT = read_raw_temperature();
    UP = read_raw_pressure();
    
    // do temperature calculations
    X1 = (UT - (INT32)AC6) * (INT32)AC5 >> 15;
    X2 = ((INT32)MC << 11) / (X1 + (INT32)MD);
    B5 = X1 + X2;
    
    // do pressure calcs
    B6 = B5 - 4000;
    X1 = ((INT32)B2 * ( (B6 * B6)>>12 )) >> 11;
    X2 = ((INT32)AC2 * B6) >> 11;
    X3 = X1 + X2;
    B3 = ((((INT32)AC1*4 + X3) << oversampling) + 2) >> 2;
    
    X1 = ((INT32)AC3 * B6) >> 13;
    X2 = ((INT32)B1 * ((B6 * B6) >> 12)) >> 16;
    X3 = ((X1 + X2) + 2) >> 2;
    B4 = ((UINT32)AC4 * (UINT32)(X3 + 32768)) >> 15;
    B7 = ((UINT32)UP - B3) * (UINT32)( 50000 >> oversampling );
    
    if (B7 < 0x80000000) {
        p = (B7 * 2) / B4;
    } else {
        p = (B7 / B4) * 2;
    }
    X1 = (p >> 8) * (p >> 8);
    X1 = (X1 * 3038) >> 16;
    X2 = (-7357 * p) >> 16;
    
    p = p + ((X1 + X2 + (INT32)3791)>>4);
    return p;
}

/*
 Returns the altitude in meters. must call temperature and pressure before
 */
float get_altitude(void) {
    float altitude;

    float pressure = get_pressure(); // in Si units for Pascal

    altitude = 44330 * (1.0 - pow(pressure /SEA_LEVEL_PRESSURE,0.1903));

    return altitude;
}

UINT16 read_raw_temperature(void) {
    //write BMP183_REGISTER_READTEMPCMD into register BMP183_REGISTER_CONTROL(0xF4))
    spiwrite(BMP183_REGISTER_CONTROL, BMP183_REGISTER_READTEMPCMD);
    //wait 5ms
    delayMs(10);
    
    //read reg BMP183_REGISTER_TEMPDATA (2 bytes - 16 bits))
    //read reg 0xF6(MSB), 0xF7(LSB)
    return spiread16(BMP183_REGISTER_TEMPDATA);
}

UINT32 read_raw_pressure(void) {
    UINT32 raw;
    //write BMP183_REGISTER_READPRESSURECMD into register BMP183_REGISTER_CONTROL(0xF4))
    spiwrite(BMP183_REGISTER_CONTROL, BMP183_REGISTER_READPRESSURECMD);
    
    if (oversampling == BMP183_MODE_ULTRALOWPOWER) 
        delayMs(5);
    else if (oversampling == BMP183_MODE_STANDARD) 
        delayMs(8);
    else if (oversampling == BMP183_MODE_HIGHRES) 
        delayMs(14);
    else 
        delayMs(26);
    
    //read reg BMP183_REGISTER_PRESSUREDATA(0xF6-0xF8) MSB-LSB (3 bytes-24 bits)
    raw = spiread16(BMP183_REGISTER_PRESSUREDATA);

    raw <<= 8;
    raw |= spiread8(BMP183_REGISTER_PRESSUREDATA+2);
    raw >>= (8 - oversampling);
    return raw;
}

void read_calibration_data(void) {
    AC1 = spiread16(BMP183_REGISTER_CAL_AC1);
    AC2 = spiread16(BMP183_REGISTER_CAL_AC2);
    AC3 = spiread16(BMP183_REGISTER_CAL_AC3);
    AC4 = spiread16(BMP183_REGISTER_CAL_AC4);
    AC5 = spiread16(BMP183_REGISTER_CAL_AC5);
    AC6 = spiread16(BMP183_REGISTER_CAL_AC6);
    B1 = spiread16(BMP183_REGISTER_CAL_B1);
    B2 = spiread16(BMP183_REGISTER_CAL_B2);
    MB = spiread16(BMP183_REGISTER_CAL_MB);
    MC = spiread16(BMP183_REGISTER_CAL_MC);
    MD = spiread16(BMP183_REGISTER_CAL_MD);
}

void delayMs(unsigned long i){
/* Create a software delay about i ms long
 * Parameters:
 *      i:  equal to number of milliseconds for delay
 * Returns: Nothing
 * Note: Uses Core Timer. Core Timer is cleared at the initialiazion of
 *      this function. So, applications sensitive to the Core Timer are going
 *      to be affected
 */
    unsigned int j;
    j = dTimeMs * i;
    WriteCoreTimer(0);
    while (ReadCoreTimer() < j);
}